@@ -3,7 +3,7 @@ use warnings;
 
 =head1 NAME
 
-    Algorithm::Evolutionary::Experiment - Class for setting up an experiment
+    Algorithm::Evolutionary::Experiment - Class for setting up an experiment with algorithms and population
                  
 =head1 SYNOPSIS
   
@@ -17,7 +17,9 @@ use warnings;
 
 =head1 DESCRIPTION
 
-Experiment contains an algorithm and a population, and applies one to the other. Contains both as instance variables.
+Experiment contains an algorithm and a population, and applies one to
+the other. Contains both as instance variables. Can be serialized
+using XML, and can read an XML description of the experiment. 
 
 =head1 METHODS
 
@@ -27,11 +29,13 @@ package Algorithm::Evolutionary::Experiment;
 
 use Algorithm::Evolutionary::Individual::Base;
 use Algorithm::Evolutionary::Op::Base;
+use Algorithm::Evolutionary::Op::Creator;
 
-our $VERSION = ( '$Revision: 1.3 $ ' =~ /(\d+\.\d+)/ ) ;
+our $VERSION = ( '$Revision: 1.8 $ ' =~ /(\d+\.\d+)/ ) ;
 
 use Carp;
-use XML::Simple;
+use XML::Parser;
+use XML::Parser::EasyTree;
 
 =head2 new
 
@@ -45,9 +49,9 @@ use XML::Simple;
 
 sub new ($$$$;$) {
   my $class = shift;
-  my $self = ();
-  $self->{_pop} = ();
-  if ( (ref $_[0]) !~ /Algorithm::Evolutionary/ ) { 
+  my $self = { _pop => [] };
+  if ( index ( ref $_[0], 'Algorithm::Evolutionary') == -1 )   {  
+    #If the first arg is not an algorithm, create one
     my $popSize = shift || carp "Pop size = 0, can't create\n";
     my $indiType = shift || carp "Empty individual class, can't create\n";
     my $indiSize = shift || carp "Empty individual size, no reasonable default, can't create\n";
@@ -57,7 +61,7 @@ sub new ($$$$;$) {
       push @{$self->{_pop}}, $indi;
     }
   };
-  @_ || die "Can't find an algorithm";
+  @_ || croak "Can't find an algorithm";
   push @{$self->{_algo}}, @_;
   bless $self, $class;
   return $self
@@ -104,31 +108,52 @@ example of it follows:
  
  This is an alternative constructor. Takes a well-formed string with the XML
  spec, which should have been done according to EvoSpec 0.3, or the same
- string processed with C<XML::Simple>, and returns a built experiment
+ string processed with C<XML::Parser::EasyTree>, and returns a built experiment
 
 =cut
 
 sub fromXML ($;$) {
   my $class = shift;
   my $xml = shift || carp "XML fragment missing ";
-  if ( (ref $xml) ne "HASH" ) { #We are receiving a string
-    $xml = XMLin($xml);
+  if ( ref $xml eq ''  ) { #We are receiving a string, parse it
+    my $p=new XML::Parser(Style=>'EasyTree');
+    $XML::Parser::EasyTree::Noempty=1;
+    $xml = $p->parse($xml);
   }
-  my $self = {}; # Create a reference
 
+  my $self = { _pop => []}; # Create a reference
   #Process population, via the creator operator
-  if ( $xml->{initial}{op}{name} ) {
-    push( @{$self->{_algo}}, Algorithm::Evolutionary::Op::Base->fromXML( $xml->{initial}{op} ) );
-  } else {
-    for ( keys  %{$xml->{initial}{op}} ) {
-      push( @{$self->{_algo}}, Algorithm::Evolutionary::Op::Base->fromXML( $xml->{initial}{op}{$_}, $_ ) );
+  for ( @{$xml->[0]{content}[0]{content}} ) { #Should process the <initial> tag
+    if ( $_->{name} eq 'pop' ) {
+      my $size = $_->{attrib}{size};
+      my $type;
+      my %params;
+      for ( @{$_->{content}} ) {
+	if ( $_->{attrib}{name} eq 'type' ) {
+	  $type = $_->{attrib}{value};
+	} else {
+	  $params{ $_->{attrib}{name} } = $_->{attrib}{value};
+	}
+      }
+      my $creator = new Algorithm::Evolutionary::Op::Creator  $size, $type, \%params ;
+      my @pop;
+      $creator->apply( \@pop );
+      push( @{$self->{_pop}}, @pop ) ;
+    }
+    if ( $_->{name} eq 'op' ) {
+      push( @{$self->{_algo}}, 
+	  Algorithm::Evolutionary::Op::Base::fromXML( $_->{attrib}{name}, $_->{content} ) );
     }
   }
 
   #Process population, if it exists
-  $self->{_pop} = [];
-  for (  @{$xml->{pop}{indi}} ) {
-	push( @{$self->{_pop}}, Algorithm::Evolutionary::Individual::Base->fromXML( $_ ) );
+  if ( $xml->[0]{content}[1]{content} ) { #Process runtime population
+    for (  @{$xml->[0]{content}[1]{content}} ) {
+      if ( $_->{name} eq 'indi' ) {
+	push( @{$self->{_pop}}, 
+	      Algorithm::Evolutionary::Individual::Base::fromXML( $_->{attrib}{type}, $_->{content} ) );
+      }
+    }
   }
   #Bless and return
   bless $self, $class;
@@ -162,11 +187,9 @@ the population
 sub asXML {
   my $self = shift;
   my $str=<<'EOC';
-<ea xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
-    xsi:noNamespaceSchemaLocation='ea-alpha.xsd'
-    version='0.3'>
+<ea version='0.4'>
 <!-- Serialization of an Experiment object. Generated automatically by
-     Experiment $Revision: 1.3 $ -->
+     Experiment $Revision: 1.8 $ -->
     <initial>
 EOC
 
@@ -187,10 +210,10 @@ EOC
   This file is released under the GPL. See the LICENSE file included in this distribution,
   or go to http://www.fsf.org/licenses/gpl.txt
 
-  CVS Info: $Date: 2002/09/01 15:27:51 $ 
-  $Header: /cvsroot/opeal/opeal/Algorithm/Evolutionary/Experiment.pm,v 1.3 2002/09/01 15:27:51 jmerelo Exp $ 
+  CVS Info: $Date: 2003/02/27 08:03:08 $ 
+  $Header: /cvsroot/opeal/opeal/Algorithm/Evolutionary/Experiment.pm,v 1.8 2003/02/27 08:03:08 jmerelo Exp $ 
   $Author: jmerelo $ 
-  $Revision: 1.3 $
+  $Revision: 1.8 $
   $Name $
 
 =cut
